@@ -12,12 +12,15 @@ import (
 	"github.com/gophercloud/gophercloud"
 	"github.com/gophercloud/gophercloud/openstack"
 	"github.com/gophercloud/gophercloud/openstack/objectstorage/v1/containers"
+	"github.com/gophercloud/gophercloud/openstack/objectstorage/v1/objects"
+	"github.com/gophercloud/gophercloud/pagination"
 	"github.com/gophercloud/utils/openstack/clientconfig"
 	"github.com/goware/urlx"
 	yamlv2 "gopkg.in/yaml.v2"
 
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
+	k8sutilerrors "k8s.io/apimachinery/pkg/util/errors"
 	"k8s.io/klog"
 
 	imageregistryv1 "github.com/openshift/api/imageregistry/v1"
@@ -400,10 +403,10 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 		// If we were supplied a container name and it exists
 		// we can skip the create
 		if !generatedName && err == nil {
-			util.UpdateCondition(cr, defaults.StorageExists, operatorapi.ConditionTrue, "Container exists", "User supplied container already exists")
-			cr.Status.Storage = imageregistryv1.ImageRegistryConfigStorage{
-				Swift: d.Config.DeepCopy(),
+			if cr.Spec.Storage.ManagementState == "" {
+				cr.Spec.Storage.ManagementState = imageregistryv1.StorageManagementStateUnmanaged
 			}
+			util.UpdateCondition(cr, defaults.StorageExists, operatorapi.ConditionTrue, "Container exists", "User supplied container already exists")
 			break
 		}
 		// If we generated a container name and it exists
@@ -423,13 +426,14 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 		_, err = containers.Create(client, cr.Spec.Storage.Swift.Container, createOps).Extract()
 		if err != nil {
 			util.UpdateCondition(cr, defaults.StorageExists, operatorapi.ConditionFalse, "Creation Failed", err.Error())
-			cr.Status.StorageManaged = false
 			return err
 		}
 
 		util.UpdateCondition(cr, defaults.StorageExists, operatorapi.ConditionTrue, "Swift Container Created", "")
 
-		cr.Status.StorageManaged = true
+		if cr.Spec.Storage.ManagementState == "" {
+			cr.Spec.Storage.ManagementState = imageregistryv1.StorageManagementStateManaged
+		}
 		cr.Status.Storage = imageregistryv1.ImageRegistryConfigStorage{
 			Swift: d.Config.DeepCopy(),
 		}
@@ -442,7 +446,8 @@ func (d *driver) CreateStorage(cr *imageregistryv1.Config) error {
 }
 
 func (d *driver) RemoveStorage(cr *imageregistryv1.Config) (bool, error) {
-	if !cr.Status.StorageManaged {
+	if cr.Spec.Storage.ManagementState != imageregistryv1.StorageManagementStateManaged ||
+		cr.Spec.Storage.Swift.Container == "" {
 		return false, nil
 	}
 
@@ -451,10 +456,43 @@ func (d *driver) RemoveStorage(cr *imageregistryv1.Config) (bool, error) {
 		return false, err
 	}
 
+	pager := objects.List(client, cr.Spec.Storage.Swift.Container, &objects.ListOpts{
+		Limit: 50,
+	})
+	if err := pager.EachPage(func(page pagination.Page) (bool, error) {
+		objectsOnPage, err := objects.ExtractNames(page)
+		if err != nil {
+			return false, err
+		}
+		resp, err := objects.BulkDelete(client, cr.Spec.Storage.Swift.Container, objectsOnPage).Extract()
+		if err != nil {
+			return false, err
+		}
+		if len(resp.Errors) > 0 {
+			// Convert resp.Errors to golang errors.
+			// Each error is represented by a list of 2 strings, where the first one
+			// is the object name, and the second one contains an error message.
+			errs := make([]error, len(resp.Errors))
+			for i, objectError := range resp.Errors {
+				errs[i] = fmt.Errorf("cannot delete object %v: %v", objectError[0], objectError[1])
+			}
+
+			return false, fmt.Errorf("errors occured during bulk deleting of container %v objects: %v", cr.Spec.Storage.Swift.Container, k8sutilerrors.NewAggregate(errs))
+		}
+
+		return true, nil
+	}); err != nil {
+		if _, ok := err.(gophercloud.ErrDefault404); !ok {
+			return false, err
+		}
+	}
+
 	_, err = containers.Delete(client, cr.Spec.Storage.Swift.Container).Extract()
 	if err != nil {
-		util.UpdateCondition(cr, defaults.StorageExists, operatorapi.ConditionUnknown, err.Error(), err.Error())
-		return false, err
+		if _, ok := err.(gophercloud.ErrDefault404); !ok {
+			util.UpdateCondition(cr, defaults.StorageExists, operatorapi.ConditionUnknown, err.Error(), err.Error())
+			return false, err
+		}
 	}
 
 	cr.Spec.Storage.Swift.Container = ""
